@@ -1,15 +1,12 @@
 r"""Functionality for storing and analyzing a year/season of cyclones."""
 
-import calendar
 import numpy as np
 import pandas as pd
-import re
-import scipy.interpolate as interp
-import urllib
 import warnings
 from datetime import datetime as dt,timedelta
 from copy import copy
 
+#Import internal scripts
 from .plot import TrackPlot
 from .storm import Storm
 
@@ -63,7 +60,7 @@ class Season:
         new_coords = self.coords.copy()
         
         #Add year to list of years
-        if isinstance(self.coords['year'],int) == True:
+        if isinstance(self.coords['year'],int):
             new_coords['year'] = [self.year,new.year]
         else:
             new_coords['year'].append(new.year)
@@ -236,34 +233,36 @@ class Season:
         """
         
         #Check if storm is str or tuple
-        if isinstance(storm, str) == True:
+        if isinstance(storm, str):
             key = storm
-        elif isinstance(storm, tuple) == True:
+        elif isinstance(storm, tuple):
             key = self.get_storm_id((storm[0],storm[1]))
         else:
             raise RuntimeError("Storm must be a string (e.g., 'AL052019') or tuple (e.g., ('Matthew',2016)).")
         
         #Retrieve key of given storm
-        if isinstance(key, str) == True:
+        if isinstance(key, str):
             return Storm(self.dict[key])
         else:
             error_message = ''.join([f"\n{i}" for i in key])
             error_message = f"Multiple IDs were identified for the requested storm. Choose one of the following storm IDs and provide it as the 'storm' argument instead of a tuple:{error_message}"
             raise RuntimeError(error_message)
         
-    def plot(self,subset=None,domain=None,ax=None,return_ax=False,cartopy_proj=None,prop={},map_prop={}):
+    def plot(self,domain=None,ax=None,cartopy_proj=None,save_path=None,prop={},map_prop={}):
         
         r"""
         Creates a plot of this season.
         
         Parameters
         ----------
+        domain : str
+            Domain for the plot. Default is basin-wide. Please refer to :ref:`options-domain` for available domain options.
         ax : axes
             Instance of axes to plot on. If none, one will be generated. Default is none.
-        return_ax : bool
-            If True, returns the axes instance on which the plot was generated for the user to further modify. Default is False.
         cartopy_proj : ccrs
             Instance of a cartopy projection to use. If none, one will be generated. Default is none.
+        save_path : str
+            Relative or full path of directory to save the image in. If none, image will not be saved.
         
         Other Parameters
         ----------------
@@ -271,16 +270,13 @@ class Season:
             Customization properties of storm track lines. Please refer to :ref:`options-prop` for available options.
         map_prop : dict
             Customization properties of Cartopy map. Please refer to :ref:`options-map-prop` for available options.
+        
+        Returns
+        -------
+        ax
+            Instance of axes containing the plot is returned.
         """
-        
-        season = copy(self)
-        stormkeys = list(season.dict.keys())
-        
-        if subset is not None:
-            subkeys = [stormskeys[i] for i in subset]
-            season.dict = {key: value for key, value in season.dict.items() if key in subkeys}
-            
-        
+
         #Create instance of plot object
         self.plot_obj = TrackPlot()
         
@@ -290,10 +286,10 @@ class Season:
             self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=0.0)
             
         #Plot storm
-        return_ax = self.plot_obj.plot_season(season,domain,ax=ax,return_ax=return_ax,prop=prop,map_prop=map_prop)
+        plot_ax = self.plot_obj.plot_season(self,domain,ax=ax,save_path=save_path,prop=prop,map_prop=map_prop)
         
         #Return axis
-        if ax != None or return_ax == True: return return_ax
+        return plot_ax
         
     def summary(self):
         
