@@ -1,6 +1,5 @@
 r"""Functionality for storing and analyzing an individual storm."""
 
-import calendar
 import numpy as np
 import pandas as pd
 import re
@@ -9,8 +8,9 @@ import urllib
 import warnings
 from datetime import datetime as dt,timedelta
 import requests
-import copy 
+import copy
 
+#Import internal scripts
 from .plot import TrackPlot
 from ..tornado import *
 from ..recon import *
@@ -69,6 +69,7 @@ class Storm:
         
         #Format keys for summary
         type_array = np.array(self.dict['type'])
+        if self.invest: idx = np.array([True for i in type_array])
         idx = np.where((type_array == 'SD') | (type_array == 'SS') | (type_array == 'TD') | (type_array == 'TS') | (type_array == 'HU'))[0]
         if len(idx) == 0:
             start_date = 'N/A'
@@ -130,6 +131,7 @@ class Storm:
             self.vars = {}
             for key in keys:
                 if key == 'realtime': continue
+                if key == 'invest': continue
                 if isinstance(self.dict[key], list) == False and isinstance(self.dict[key], dict) == False:
                     self[key] = self.dict[key]
                     self.coords[key] = self.dict[key]
@@ -138,7 +140,7 @@ class Storm:
                     self[key] = np.array(self.dict[key])
 
             #Assign tornado data
-            if stormTors != None and isinstance(stormTors,dict) == True:
+            if stormTors is not None and isinstance(stormTors,dict):
                 self.stormTors = stormTors['data']
                 self.tornado_dist_thresh = stormTors['dist_thresh']
                 self.coords['Tornado Count'] = len(stormTors['data'])
@@ -150,12 +152,20 @@ class Storm:
                 pass
 
             #Determine if storm object was retrieved via realtime object
-            if 'realtime' in keys and self.dict['realtime'] == True:
+            if 'realtime' in keys and self.dict['realtime']:
                 self.realtime = True
                 self.coords['realtime'] = True
             else:
                 self.realtime = False
                 self.coords['realtime'] = False
+            
+            #Determine if storm object is an invest
+            if 'invest' in keys and self.dict['invest']:
+                self.invest = True
+                self.coords['invest'] = True
+            else:
+                self.invest = False
+                self.coords['invest'] = False
         
         else:
             
@@ -453,7 +463,7 @@ class Storm:
 
         #Construct new storm dict with subset elements
         for key in NEW_STORM.dict.keys():
-            if isinstance(NEW_STORM.dict[key], list) == True:
+            if isinstance(NEW_STORM.dict[key], list):
                 NEW_STORM.dict[key] = [NEW_STORM.dict[key][i] for i in idx_final]
             else:
                 NEW_STORM.dict[key] = NEW_STORM.dict[key]
@@ -485,8 +495,8 @@ class Storm:
         
         Returns
         -------
-        storm object
-            New storm object containing the updated dictionary.
+        tropycal.tracks.Storm
+            New Storm object containing the updated dictionary.
         """
         
         NEW_STORM = copy.copy(self)
@@ -546,7 +556,7 @@ class Storm:
         #Add every key containing a list into the dict, otherwise add as an attribute
         keys = [k for k in self.dict.keys() if k != 'date']
         for key in keys:
-            if isinstance(self.dict[key], list) == True:
+            if isinstance(self.dict[key], list):
                 ds[key] = xr.DataArray(self.dict[key],coords=[time],dims=['time'])
             else:
                 attrs[key] = self.dict[key]
@@ -557,11 +567,16 @@ class Storm:
         #Return dataset
         return ds
 
-    def to_dataframe(self):
+    def to_dataframe(self, attrs_as_columns=False):
         
         r"""
         Converts the storm dict into a pandas DataFrame object.
         
+        Parameters
+        ----------
+        attrs_as_columns : bool
+            If True, adds Storm object attributes as columns in the DataFrame returned. Default is False.
+        
         Returns
         -------
         pandas.DataFrame
@@ -581,17 +596,19 @@ class Storm:
         #Add every key containing a list into the dict
         keys = [k for k in self.dict.keys()]
         for key in keys:
-            if isinstance(self.dict[key], list) == True:
+            if isinstance(self.dict[key], list):
                 ds[key] = self.dict[key]
-                    
+            else:
+                if attrs_as_columns:
+                    ds[key] = self.dict[key]
+
         #Convert entire dict to a DataFrame
         ds = pd.DataFrame(ds)
 
         #Return dataset
         return ds
     
-    #PLOT FUNCTION FOR HURDAT
-    def plot(self,domain="dynamic",plot_all=False,ax=None,return_ax=False,cartopy_proj=None,save_path=None,prop={},map_prop={}):
+    def plot(self,domain="dynamic",plot_all_dots=False,ax=None,cartopy_proj=None,save_path=None,prop={},map_prop={}):
         
         r"""
         Creates a plot of the observed track of the storm.
@@ -600,12 +617,10 @@ class Storm:
         ----------
         domain : str
             Domain for the plot. Default is "dynamic". "dynamic_tropical" is also available. Please refer to :ref:`options-domain` for available domain options.
-        plot_all : bool
+        plot_all_dots : bool
             Whether to plot dots for all observations along the track. If false, dots will be plotted every 6 hours. Default is false.
         ax : axes
             Instance of axes to plot on. If none, one will be generated. Default is none.
-        return_ax : bool
-            If True, returns the axes instance on which the plot was generated for the user to further modify. Default is False.
         cartopy_proj : ccrs
             Instance of a cartopy projection to use. If none, one will be generated. Default is none.
         save_path : str
@@ -617,6 +632,11 @@ class Storm:
             Customization properties of storm track lines. Please refer to :ref:`options-prop` for available options.
         map_prop : dict
             Customization properties of Cartopy map. Please refer to :ref:`options-map-prop` for available options.
+        
+        Returns
+        -------
+        ax
+            Instance of axes containing the plot is returned.
         """
         
         #Create instance of plot object
@@ -626,23 +646,22 @@ class Storm:
             self.plot_obj = TrackPlot()
         
         #Create cartopy projection
-        if cartopy_proj == None:
-            if max(self.dict['lon']) > 150 or min(self.dict['lon']) < -150:
-                self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=180.0)
-            else:
-                self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=0.0)
-        else:
+        if cartopy_proj is not None:
             self.plot_obj.proj = cartopy_proj
+        elif max(self.dict['lon']) > 150 or min(self.dict['lon']) < -150:
+            self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=180.0)
+        else:
+            self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=0.0)
             
         #Plot storm
-        plot_ax = self.plot_obj.plot_storm(self.dict,domain,plot_all,ax=ax,return_ax=return_ax,prop=prop,map_prop=map_prop,save_path=save_path)
+        plot_ax = self.plot_obj.plot_storm(self.dict,domain,plot_all_dots,ax=ax,prop=prop,map_prop=map_prop,save_path=save_path)
         
         #Return axis
-        if ax != None or return_ax == True: return plot_ax
+        return plot_ax
         
     #PLOT FUNCTION FOR HURDAT
     def plot_nhc_forecast(self,forecast,track_labels='fhr',cone_days=5,domain="dynamic_forecast",
-                          ax=None,return_ax=False,cartopy_proj=None,save_path=None,prop={},map_prop={}):
+                          ax=None,cartopy_proj=None,save_path=None,prop={},map_prop={}):
         
         r"""
         Creates a plot of the operational NHC forecast track along with observed track data.
@@ -664,8 +683,6 @@ class Storm:
             Domain for the plot. Default is "dynamic_forecast". Please refer to :ref:`options-domain` for available domain options.
         ax : axes
             Instance of axes to plot on. If none, one will be generated. Default is none.
-        return_ax : bool
-            If True, returns the axes instance on which the plot was generated for the user to further modify. Default is False.
         cartopy_proj : ccrs
             Instance of a cartopy projection to use. If none, one will be generated. Default is none.
         save_path : str
@@ -677,12 +694,21 @@ class Storm:
             Customization properties of NHC forecast plot. Please refer to :ref:`options-prop-nhc` for available options.
         map_prop : dict
             Customization properties of Cartopy map. Please refer to :ref:`options-map-prop` for available options.
+        
+        Returns
+        -------
+        ax
+            Instance of axes containing the plot is returned.
         """
         
         #Check to ensure the data source is HURDAT
         if self.source != "hurdat":
             raise RuntimeError("Error: NHC data can only be accessed when HURDAT is used as the data source.")
         
+        #Check to ensure storm is not an invest
+        if self.invest:
+            raise RuntimeError("Error: NHC does not issue advisories for invests that have not been designated as Potential Tropical Cyclones.")
+        
         #Create instance of plot object
         try:
             self.plot_obj
@@ -690,7 +716,7 @@ class Storm:
             self.plot_obj = TrackPlot()
         
         #Create cartopy projection
-        if cartopy_proj == None:
+        if cartopy_proj is None:
             if max(self.dict['lon']) > 140 or min(self.dict['lon']) < -140:
                 self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=180.0)
             else:
@@ -711,10 +737,10 @@ class Storm:
         carq_forecast_init = [k for k in carq_forecasts.keys()]
 
         #Find closest matching time to the provided forecast date, or time
-        if isinstance(forecast,int) == True:
+        if isinstance(forecast,int):
             forecast_dict = nhc_forecasts[nhc_forecast_init[forecast-1]]
             advisory_num = forecast+0
-        elif isinstance(forecast,dt) == True:
+        elif isinstance(forecast,dt):
             nhc_forecast_init_dt = [dt.strptime(k,'%Y%m%d%H') for k in nhc_forecast_init]
             time_diff = np.array([(i-forecast).days + (i-forecast).seconds/86400 for i in nhc_forecast_init_dt])
             closest_idx = np.abs(time_diff).argmin()
@@ -801,10 +827,10 @@ class Storm:
         forecast_dict['basin'] = self.basin
         
         #Plot storm
-        plot_ax = self.plot_obj.plot_storm_nhc(forecast_dict,track_dict,track_labels,cone_days,domain,ax=ax,return_ax=return_ax,save_path=save_path,prop=prop,map_prop=map_prop)
+        plot_ax = self.plot_obj.plot_storm_nhc(forecast_dict,track_dict,track_labels,cone_days,domain,ax=ax,save_path=save_path,prop=prop,map_prop=map_prop)
         
         #Return axis
-        if ax != None or return_ax == True: return plot_ax
+        return plot_ax
         
     
     #PLOT FUNCTION FOR HURDAT
@@ -814,7 +840,7 @@ class Storm:
                             prop_gfs = {'linewidth':2.0, 'linecolor':'b'},
                             prop_ellipse = {'linewidth':2.0, 'linecolor':'r'},
                             prop_density = {'radius':200, 'cmap':plt.cm.YlOrRd, 'levels':[i for i in range(5,105,5)]},
-                            domain="dynamic",ax=None,return_ax=False,cartopy_proj=None,save_path=None,map_prop={}):
+                            domain="dynamic",ax=None,cartopy_proj=None,save_path=None,map_prop={}):
         
         r"""
         (Add track history like we do for NHC forecasts)
@@ -834,8 +860,6 @@ class Storm:
             Domain for the plot. Default is "dynamic". Please refer to :ref:`options-domain` for available domain options.
         ax : axes
             Instance of axes to plot on. If none, one will be generated. Default is none.
-        return_ax : bool
-            If True, returns the axes instance on which the plot was generated for the user to further modify. Default is False.
         cartopy_proj : ccrs
             Instance of a cartopy projection to use. If none, one will be generated. Default is none.
         save_path : str
@@ -847,6 +871,11 @@ class Storm:
             Customization properties of storm track lines. Please refer to :ref:`options-prop` for available options.
         map_prop : dict
             Customization properties of Cartopy map. Please refer to :ref:`options-map-prop` for available options.
+        
+        Returns
+        -------
+        ax
+            Instance of axes containing the plot is returned.
         """
         
         #Create instance of plot object
@@ -856,13 +885,12 @@ class Storm:
             self.plot_obj = TrackPlot()
         
         #Create cartopy projection
-        if cartopy_proj == None:
-            if max(self.dict['lon']) > 150 or min(self.dict['lon']) < -150:
-                self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=180.0)
-            else:
-                self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=0.0)
-        else:
+        if cartopy_proj is not None:
             self.plot_obj.proj = cartopy_proj
+        elif max(self.dict['lon']) > 150 or min(self.dict['lon']) < -150:
+            self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=180.0)
+        else:
+            self.plot_obj.create_cartopy(proj='PlateCarree',central_longitude=0.0)
         
         #-------------------------------------------------------------------------
         
@@ -954,7 +982,7 @@ class Storm:
                 ds['gefs']['members'].append(len(temp_data['lat']))
 
                 #Calculate ellipse data
-                if prop_ellipse != None:
+                if prop_ellipse is not None:
                     ellipse_data = plot_ellipse(temp_data['lat'],temp_data['lon'])
                     ds['gefs']['ellipse_lon'].append(ellipse_data['xell'])
                     ds['gefs']['ellipse_lat'].append(ellipse_data['yell'])
@@ -964,10 +992,10 @@ class Storm:
             fhr = [fhr]
         
         #Plot storm
-        plot_ax = self.plot_obj.plot_ensembles(forecast,self.dict,fhr,prop_members,prop_mean,prop_gfs,prop_ellipse,prop_density,nens,domain,ds,ax=ax,return_ax=return_ax,map_prop=map_prop,save_path=save_path)
+        plot_ax = self.plot_obj.plot_ensembles(forecast,self.dict,fhr,prop_members,prop_mean,prop_gfs,prop_ellipse,prop_density,nens,domain,ds,ax=ax,map_prop=map_prop,save_path=save_path)
         
         #Return axis
-        if ax != None or return_ax == True: return plot_ax
+        return plot_ax
     
     def list_nhc_discussions(self):
         
@@ -984,6 +1012,10 @@ class Storm:
         if self.source != "hurdat":
             raise RuntimeError("Error: NHC data can only be accessed when HURDAT is used as the data source.")
         
+        #Check to ensure storm is not an invest
+        if self.invest:
+            raise RuntimeError("Error: NHC does not issue advisories for invests that have not been designated as Potential Tropical Cyclones.")
+        
         #Get storm ID & corresponding data URL
         storm_id = self.dict['operational_id']
         storm_year = self.dict['year']
@@ -1303,6 +1335,10 @@ class Storm:
             msg = "Error: NHC data can only be accessed when HURDAT is used as the data source."
             raise RuntimeError(msg)
         
+        #Check to ensure storm is not an invest
+        if self.invest:
+            raise RuntimeError("Error: NHC does not issue advisories for invests that have not been designated as Potential Tropical Cyclones.")
+        
         #Get storm ID & corresponding data URL
         storm_id = self.dict['operational_id']
         storm_year = self.dict['year']
@@ -1320,7 +1356,7 @@ class Storm:
         #Get list of storm discussions
         disco_dict = self.list_nhc_discussions()
         
-        if isinstance(forecast,dt) == True:
+        if isinstance(forecast,dt):
             #Find closest discussion to the time provided
             disco_times = disco_dict['utc_date']
             disco_ids = [int(i) for i in disco_dict['id']]
@@ -1334,7 +1370,7 @@ class Storm:
             if np.abs(closest_diff) >= 1.0:
                 warnings.warn(f"The date provided is unavailable or outside of the duration of the storm. Use the \"list_nhc_discussions()\" function to retrieve a list of available NHC discussions for this storm. Returning the closest available NHC discussion.")
                 
-        if isinstance(forecast,int) == True:
+        if isinstance(forecast,int):
             #Find closest discussion ID to the one provided
             disco_times = disco_dict['utc_date']
             disco_ids = [int(i) for i in disco_dict['id']]
@@ -1401,7 +1437,7 @@ class Storm:
             response.close()
         
         #Save file, if specified
-        if save_path != None:
+        if save_path is not None:
             closest_time = disco_times[closest_idx].strftime("%Y%m%d_%H%M")
             fname = f"nhc_disco_{self.name.lower()}_{self.year}_{closest_time}.txt"
             o = open(save_path+fname,"w")
@@ -1433,6 +1469,10 @@ class Storm:
             msg = "Error: NHC data can only be accessed when HURDAT is used as the data source."
             raise RuntimeError(msg)
         
+        #Check to ensure storm is not an invest
+        if self.invest:
+            raise RuntimeError("Error: NHC does not issue advisories for invests that have not been designated as Potential Tropical Cyclones.")
+        
         #Get storm ID & corresponding data URL
         storm_id = self.dict['operational_id']
         storm_year = self.dict['year']
@@ -1444,7 +1484,7 @@ class Storm:
         if isinstance(query,str) == False and isinstance(query,list) == False:
             msg = "'query' must be of type str or list."
             raise TypeError(msg)
-        if isinstance(query,list) == True:
+        if isinstance(query,list):
             for i in query:
                 if isinstance(i,str) == False:
                     msg = "Entries of list 'query' must be of type str."
@@ -1464,13 +1504,13 @@ class Storm:
             text = forecast['text'].lower()
             
             #If found, add into list
-            if isinstance(query,str) == True:
+            if isinstance(query,str):
                 if text.find(query.lower()) >= 0: output.append(forecast)
             else:
                 found = False
                 for i_query in query:
                     if text.find(i_query.lower()) >= 0: found = True
-                if found == True: output.append(forecast)
+                if found: output.append(forecast)
             
         #Return list
         return output
@@ -1631,6 +1671,10 @@ class Storm:
             Path of directory to download the TCR into. Default is current working directory.
         """
         
+        #Check to ensure storm is not an invest
+        if self.invest:
+            raise RuntimeError("Error: NHC does not issue advisories for invests that have not been designated as Potential Tropical Cyclones.")
+        
         #Error check
         if self.source != "hurdat":
             msg = "NHC data can only be accessed when HURDAT is used as the data source."
@@ -1661,7 +1705,7 @@ class Storm:
 
             
     def plot_tors(self,dist_thresh=1000,Tors=None,domain="dynamic",plotPPH=False,plot_all=False,\
-                  ax=None,return_ax=False,cartopy_proj=None,prop={},map_prop={}):
+                  ax=None,cartopy_proj=None,save_path=None,prop={},map_prop={}):
                 
         r"""
         Creates a plot of the storm and associated tornado tracks.
@@ -1686,10 +1730,10 @@ class Storm:
             Whether to plot dots for all observations along the track. If false, dots will be plotted every 6 hours. Default is false.
         ax : axes
             Instance of axes to plot on. If none, one will be generated. Default is none.
-        return_ax : bool
-            If True, returns the axes instance on which the plot was generated for the user to further modify. Default is False.
         cartopy_proj : ccrs
             Instance of a cartopy projection to use. If none, one will be generated. Default is none.
+        save_path : str
+            Relative or full path of directory to save the image in. If none, image will not be saved.
         
         Other Parameters
         ----------------
@@ -1697,6 +1741,11 @@ class Storm:
             Customization properties of plot.
         map_prop : dict
             Customization properties of Cartopy map. Please refer to :ref:`options-map-prop` for available options.
+        
+        Returns
+        -------
+        ax
+            Instance of axes containing the plot is returned.
         """
         
         #Set default colormap for TC plots to Wistia
@@ -1705,7 +1754,7 @@ class Storm:
         except:
             prop['PPHcolors']='Wistia'
         
-        if Tors == None:
+        if Tors is None:
             try:
                 self.stormTors
             except:
@@ -1733,7 +1782,7 @@ class Storm:
             self.plot_obj_tor = TornadoPlot()
         
         #Create cartopy projection
-        if cartopy_proj == None:
+        if cartopy_proj is None:
             if max(self.dict['lon']) > 150 or min(self.dict['lon']) < -150:
                 self.plot_obj_tor.create_cartopy(proj='PlateCarree',central_longitude=180.0)
                 self.plot_obj_tc.create_cartopy(proj='PlateCarree',central_longitude=180.0)
@@ -1754,15 +1803,14 @@ class Storm:
         storm_title = plot_ax.get_title('left')
         plot_ax.set_title(f'{storm_title}\n{tor_title}',loc='left',fontsize=17,fontweight='bold')
         
+        #Save plot
+        if save_path is not None and isinstance(save_path,str):
+            plt.savefig(save_path,bbox_inches='tight')
+        
         #Return axis
-        if ax != None or return_ax == True: 
-            return plot_ax
-        else:
-            plt.show()
-            plt.close()
+        return plot_ax
 
-
-    def plot_TCtors_rotated(self,dist_thresh=1000,return_ax=False):
+    def plot_TCtors_rotated(self,dist_thresh=1000,save_path=None):
         
         r"""
         Plot tracks of tornadoes relative to the storm motion vector of the tropical cyclone.
@@ -1771,8 +1819,13 @@ class Storm:
         ----------
         dist_thresh : int
             Distance threshold (in kilometers) from the tropical cyclone track over which to attribute tornadoes to the TC. Default is 1000 km. Ignored if tornado data was passed into Storm from TrackDataset.
-        return_ax : bool
-            If True, returns the axes instance on which the plot was generated for the user to further modify. Default is False.
+        save_path : str
+            Relative or full path of directory to save the image in. If none, image will not be saved.
+        
+        Returns
+        -------
+        ax
+            Instance of axes containing the plot is returned.
         
         Notes
         -----
@@ -1833,12 +1886,12 @@ class Storm:
         ax.text(0.99,0.01,plot_credit(),fontsize=8,color='k',alpha=0.7,
                 transform=ax.transAxes,ha='right',va='bottom',zorder=10)
         
+        #Save plot
+        if save_path is not None and isinstance(save_path,str):
+            plt.savefig(save_path,bbox_inches='tight')
+        
         #Return axis or show figure
-        if return_ax == True:
-            return ax
-        else:
-            plt.show()
-            plt.close()
+        return ax
             
     def get_recon(self,deltap_thresh=8,save_path="",read_path="",mission_url_list=None,update=False):
         
