@@ -1,4 +1,5 @@
 import os, sys
+import math
 import numpy as np
 import pandas as pd
 from datetime import datetime as dt,timedelta
@@ -34,7 +35,10 @@ def find_var(request,thresh):
     
     #Count of number of storms
     if request.find('count') >= 0 or request.find('num') >= 0:
-        return thresh, 'date' #not sure what date stands for
+        return thresh, 'type'
+
+    if request.find('date') >= 0 or request.find('day') >= 0:
+        return thresh, 'date'
     
     #Sustained wind, or change in wind speed
     if request.find('wind') >= 0 or request.find('vmax') >= 0:
@@ -50,9 +54,11 @@ def find_var(request,thresh):
         else:
             return thresh,'vmax'
 
-    elif request.find('ace')>=0 or request.find('accumulated')>=0:
+    elif request.find('ace')>=0:
         return thresh,'ace'
-        
+    elif request.find('acie')>=0:
+        return thresh,'acie'
+    
     #Minimum MSLP, or change in MSLP
     elif request.find('pressure') >= 0 or request.find('slp') >= 0:
         #If change in MSLP, determine time interval
@@ -106,11 +112,11 @@ def find_func(request,thresh):
     request = request.lower()
     
     #Numpy maximum function
-    if request.find('max') == 0:
+    if request.find('max') == 0 or request.find('latest') == 0:
         return thresh, lambda x: np.nanmax(x)
     
     #Numpy minimum function
-    if request.find('min') == 0:
+    if request.find('min') == 0 or request.find('earliest') == 0:
         return thresh, lambda x: np.nanmin(x)
     
     #Numpy average function
@@ -129,7 +135,9 @@ def find_func(request,thresh):
         return thresh, lambda x: len(x)
     
     #ACE - cumulative function
-    elif request.find('ace') >=0 or request.find('accumulated') >=0:
+    elif request.find('ace') >=0:
+        return thresh, lambda x: np.nansum(x)
+    elif request.find('acie') >=0:
         return thresh, lambda x: np.nansum(x)
     
     #Otherwise, function cannot be identified
@@ -255,6 +263,15 @@ def interp_storm(storm_dict,timeres=1,dt_window=24,dt_align='middle'):
     storm_dict['type'] = np.asarray(storm_dict['type'])
     storm_dict['lon'] = np.array(storm_dict['lon']) % 360
     
+    def round_datetime(tm,nearest_minute=10):
+        discard = timedelta(minutes=tm.minute % nearest_minute,
+                             seconds=tm.second,
+                             microseconds=tm.microsecond)
+        tm -= discard
+        if discard >= timedelta(minutes=int(nearest_minute/2)):
+            tm += timedelta(minutes=nearest_minute)
+        return tm
+    
     #Attempt temporal interpolation
     try:
         
@@ -262,18 +279,41 @@ def interp_storm(storm_dict,timeres=1,dt_window=24,dt_align='middle'):
         targettimes = np.arange(times[0],times[-1]+timeres/24,timeres/24)
         
         #Update dates
-        new_storm['date'] = [t.replace(tzinfo=None) for t in mdates.num2date(targettimes)]
+        new_storm['date'] = [round_datetime(t.replace(tzinfo=None)) for t in mdates.num2date(targettimes)]
+        targettimes = mdates.date2num(np.array(new_storm['date']))
+        
+        #Create same-length lists for other things
+        new_storm['special'] = ['']*len(new_storm['date'])
+        new_storm['extra_obs'] = [0]*len(new_storm['date'])
+        
+        #WMO basin. Simple linear interpolation.
+        basinnum = np.cumsum([0]+[1 if storm_dict['wmo_basin'][i+1]!=j else 0\
+                    for i,j in enumerate(storm_dict['wmo_basin'][:-1])])
+        basindict = {k:v for k,v in zip(basinnum,storm_dict['wmo_basin'])}
+        basininterp = np.round(np.interp(targettimes,times,basinnum)).astype(int)
+        new_storm['wmo_basin'] = [basindict[k] for k in basininterp]
         
         #Interpolate and fill in storm type
-        stormtype = np.ones(len(storm_dict['type']))*-99
-        stormtype[np.where((storm_dict['type']=='TD') | (storm_dict['type']=='SD') | (storm_dict['type']=='TS') | \
-                           (storm_dict['type']=='SS') | (storm_dict['type']=='HU'))] = 0
-        new_storm['type'] = np.interp(targettimes,times,stormtype)
-        new_storm['type'] = np.where(new_storm['type']<0,'NT','TD')
+        stormtype = [1 if i in ('TD','SD','TS','SS','HU') else -1 for i in storm_dict['type']]
+        isTROP = np.interp(targettimes,times,stormtype)
+        stormtype = [1 if i in ('SD','SS') else -1 for i in storm_dict['type']]
+        isSUB = np.interp(targettimes,times,stormtype)
+        stormtype = [1 if i=='LO' else -1 for i in storm_dict['type']]
+        isLO = np.interp(targettimes,times,stormtype)
+        stormtype = [1 if i=='DB' else -1 for i in storm_dict['type']]
+        isDB = np.interp(targettimes,times,stormtype)
+        newtype = np.where(isTROP>0,'TROP','EX')
+        newtype[newtype=='TROP'] = np.where(isSUB[newtype=='TROP']>0,'SUB','TROP')
+        newtype[newtype=='EX'] = np.where(isLO[newtype=='EX']>0,'LO','EX')
+        newtype[newtype=='EX'] = np.where(isDB[newtype=='EX']>0,'DB','EX')
         
         #Interpolate and fill in other variables
         for name in ['vmax','mslp','lat','lon']:
             new_storm[name] = np.interp(targettimes,times,storm_dict[name])
+        #Correct storm type by intensity
+        newtype[newtype=='TROP'] = [['TD','TS','HU'][int(i>34)+int(i>63)] for i in new_storm['vmax'][newtype=='TROP']]
+        newtype[newtype=='SUB'] = [['SD','SS'][int(i>34)] for i in new_storm['vmax'][newtype=='SUB']]
+        new_storm['type'] = newtype
         
         #Calculate change in wind & MSLP over temporal resolution
         new_storm['dvmax_dt'] = [np.nan] + list((new_storm['vmax'][1:]-new_storm['vmax'][:-1]) / timeres)
@@ -310,6 +350,9 @@ def interp_storm(storm_dict,timeres=1,dt_window=24,dt_align='middle'):
             if dt_align=='start':
                 new_storm[name] = list(tmp)+[np.nan]*(len(new_storm[name])-len(tmp))
         
+        new_storm['dt_window'] = dt_window
+        new_storm['dt_align'] = dt_align
+        
         #Return new dict
         return new_storm
     
@@ -346,7 +389,7 @@ def filter_storms_vp(trackdata,year_min=0,year_max=9999,subset_domain=None):
     """
     
     #If no subset domain is passed, use global data
-    if subset_domain == None:
+    if subset_domain is None:
         lon_min,lon_max,lat_min,lat_max = [0,360,-90,90]
     else:
         lon_min,lon_max,lat_min,lat_max = [float(i) for i in subset_domain.split("/")]
@@ -632,3 +675,236 @@ def num_to_text(number):
     else:
         msg = "Please choose a number less than 100."
         raise ValueError(msg)
+
+
+
+
+
+
+
+
+
+
+
+
+
+def calculate_distance(lat1,lat2,lon1,lon2):
+    r_earth = 6.371 * 10**6
+    
+    #=ACOS(COS(RADIANS(90-Lat1)) *COS(RADIANS(90-Lat2)) +SIN(RADIANS(90-Lat1)) *SIN(RADIANS(90-Lat2)) *COS(RADIANS(Long1-Long2))) *6371
+    step1 = math.cos(math.radians(90.0-lat1))
+    step2 = math.cos(math.radians(90.0-lat2))
+    step3 = math.sin(math.radians(90.0-lat1))
+    step4 = math.sin(math.radians(90.0-lat2))
+    step5 = math.cos(math.radians(lon1-lon2))
+    dist = math.acos(step1 * step2 + step3 * step4 * step5) * r_earth
+    
+    return dist
+
+def plot_ellipse(slats,slons):
+
+    mlon = np.average(slons)
+    mlat = np.average(slats)
+
+    Pb = [[0,0],[0,0]]
+    Pb[0][0] = 0
+    Pb[1][1] = 0
+    Pb[1][0] = 0
+
+    for i in np.arange(len(slats)):
+        Pb[0][0] = Pb[0][0] + (slons[i]-mlon) * (slons[i]-mlon)
+        Pb[1][1] = Pb[1][1] + (slats[i]-mlat) * (slats[i]-mlat)
+        Pb[1][0] = Pb[1][0] + (slats[i]-mlat) * (slons[i]-mlon)
+
+    Pb[0][1] = Pb[1][0]
+
+    Pb[0][0] = Pb[0][0] / float(len(slats)-1)
+    Pb[0][1] = Pb[0][1] / float(len(slats)-1)
+    Pb[1][0] = Pb[1][0] / float(len(slats)-1)
+    Pb[1][1] = Pb[1][1] / float(len(slats)-1)
+
+    #Pb(:,:) = Pb(:,:) / int2flt(pcnt-1)
+    #print(Pb)
+
+    rho = Pb[1][0] / (math.sqrt(Pb[0][0]) * math.sqrt(Pb[1][1]))
+    sigmax = math.sqrt(Pb[0][0])
+    sigmay = math.sqrt(Pb[1][1])
+    fac = 1.0 / (2.0 * (1 - rho * rho))
+
+    xell = []
+    yell = []
+    rdex = 0
+    incr = math.pi/180.0
+    for radians in np.arange(0,(2.0*math.pi)+incr,math.pi/180.0):
+        xstart = math.cos(radians)
+        ystart = math.sin(radians)
+
+        for rdistance in np.arange(0.,2400.):
+            xloc = xstart * rdistance/80.0
+            yloc = ystart * rdistance/80.0
+            dist = math.sqrt(xloc * xloc + yloc * yloc)
+            prob = np.exp(-1.0 * fac * ((xloc/sigmax)**2 + (yloc/sigmay)**2 - 2.0 * rho * (xloc/sigmax)*(yloc/sigmay)))
+            if prob < 0.256:
+                xell.append(xloc + mlon)
+                yell.append(yloc + mlat)
+                break
+
+    #Find edge of ellipses
+    #"""
+    edge_lon = 0
+    edge_lat = 0
+    maxdist = 0
+    for i in np.arange(len(xell)):
+        ix = xell[i]
+        iy = yell[i]
+        dist = calculate_distance(mlat,iy,mlon,ix)
+        if dist > maxdist:
+            maxdist = dist
+            edge_lon = ix
+            edge_lat = iy
+
+    near_lon = 0
+    near_lat = 0
+    mindist = 9999999
+    for i in np.arange(len(xell)):
+        ix = xell[i]
+        iy = yell[i]
+        dist = calculate_distance(mlat,iy,mlon,ix)
+        if dist < mindist:
+            mindist = dist
+            near_lon = ix
+            near_lat = iy
+    #"""
+
+    #Return ellipse data
+    return {'xell':xell,'yell':yell,'maxdist':maxdist,'mindist':mindist}
+
+def calc_ellipse(elavec,elovec):
+    nens = len(elavec)
+
+    deg2rad   = math.atan(1.0) / 45.0
+    deg2km    = 6370.0 * deg2rad
+    nmiles2km = 1.852
+    knots2ms  = 0.5144
+
+    #evitals is vitals (ensemble member, f
+
+    #elavec        = [38.0,39.0,38.1,39.5,40.2,39.3]
+    #elovec        = [80.0,75.3,76.8,76.4,74.9,78.3]
+
+    #Arbitrarily defining these as they will be filled later
+    fxdir         = [0.0 for x in range(0,nens)]
+    fydir         = [0.0 for x in range(0,nens)]
+
+    mlat          = 0.0
+    mlon          = 0.0
+    xmean         = 0.0
+    ymean         = 0.0
+    xvar          = 0.0
+    yvar          = 0.0
+    xycov         = 0.0
+    nused         = 0.0
+
+    for n in range(0,nens):
+        nused = nused + 1
+        mlat = mlat + elavec[n]
+        mlon = mlon + elovec[n]
+
+    if nused > 1:
+
+        mlat = mlat / float(nused)
+        mlon = mlon / float(nused)
+
+        for n in range(0,nens):
+            fxdir[n] = (elovec[n]-mlon) * deg2km * math.cos(0.5*(elavec[n]+mlat)*deg2rad)
+            fydir[n] = (elavec[n]-mlat) * deg2km
+            xmean    = xmean + fxdir[n]
+            ymean    = ymean + fydir[n]
+
+        xmean = xmean / float(nused)
+        ymean = ymean / float(nused)
+
+        for n in range(0,nens):
+            xvar  = xvar  + (fxdir[n]-xmean)**2
+            yvar  = yvar  + (fydir[n]-ymean)**2
+            xycov = xycov + (fxdir[n]-xmean)*(fydir[n]-ymean)
+
+        xvar   = xvar  / float(nused-1)
+        yvar   = yvar  / float(nused-1)
+        xycov  = xycov / float(nused-1)
+        mtrace = xvar + yvar
+        mdet   = xvar * yvar - xycov * xycov
+
+        eval   = max([0.5 * mtrace + math.sqrt((mtrace * mtrace) / 4.0 - mdet),
+                      0.5 * mtrace - math.sqrt((mtrace * mtrace) / 4.0 - mdet)])
+
+        if abs(xycov) > 0:
+            #print 'here'
+            imaj = eval-yvar
+            jmaj = xycov
+        else:
+            #print 'here'
+            imaj = 1.0
+            jmaj = 0.0
+
+        vlen = math.sqrt(imaj * imaj + jmaj * jmaj)
+        imaj = imaj / vlen
+        jmaj = jmaj / vlen
+
+        imin =  jmaj
+        jmin = -imaj
+
+    return imaj,imin,jmaj,jmin,mlat,mlon
+
+def add_radius(lats,lons,vlat,vlon,rad):
+    
+    #Define empty array
+    return_arr = np.zeros((lats.shape))
+    
+    #Calculate distance from vlat/vlon at each gridpoint
+    r_earth = 6.371 * 10**6
+    dlat = np.subtract(np.radians(lats),np.radians(vlat))
+    dlon = np.subtract(np.radians(lons),np.radians(vlon))
+
+    a = np.sin(dlat/2) * np.sin(dlat/2) + np.cos(np.radians(lats)) * np.cos(np.radians(vlat)) * np.sin(dlon/2) * np.sin(dlon/2)
+    c = 2 * np.arctan(np.sqrt(a), np.sqrt(1-a));
+    dist = (r_earth * c)/1000.0
+    
+    #Mask out values less than radius
+    return_arr[dist > rad] = 0
+    return_arr[dist <= rad] = 1
+    return return_arr
+
+def listify(x):
+    if isinstance(x,(tuple,list,np.ndarray)):
+        return [i for i in x]
+    else:
+        return [x]
+    
+def make_var_label(x,storm_dict):
+    delta = u"\u0394"
+    x = list(x)
+    if x[0]=='d' and x[-3:-1]==['_','d']:
+        x[0] = delta; del x[-3:]
+        x.append(f' / {storm_dict["dt_window"]}hr')
+    x = ''.join(x)
+    if 'mslp' in x:
+        x=x.replace('mslp','mslp (hPa)')
+    if 'vmax' in x:
+        x=x.replace('vmax','vmax (kt)')
+    if 'speed' in x:
+        x=x.replace('speed','speed (kt)')    
+    return ''.join(x)
+
+def date_diff(a,b):
+    if isinstance(a,np.datetime64):
+        a=dt.utcfromtimestamp((a - np.datetime64('1970-01-01T00:00:00Z')) / np.timedelta64(1, 's'))
+    if isinstance(b,np.datetime64):
+        b=dt.utcfromtimestamp((b - np.datetime64('1970-01-01T00:00:00Z')) / np.timedelta64(1, 's'))
+    c = a.replace(year=2000)-b.replace(year=2000)
+    if c<timedelta(0):
+        try:
+            c = a.replace(year=2001)-b.replace(year=2000)
+        except:
+            c = a.replace(year=2000)-b.replace(year=1999)
+    return c
